@@ -37,6 +37,9 @@ from django_mako_plus.controller.router import MakoTemplateRenderer
 import os, os.path, time
 
 
+DMP_ATTR_NAME = 'dmp_templateinfo'  # used to attach TemplateInfo objects to Mako templates
+
+
 # Import minification if requested
 JSMIN = False
 CSSMIN = False
@@ -54,21 +57,16 @@ if settings.DMP_MINIFY_JS_CSS and not settings.DEBUG:
 
 
 ######################################################################
-###  Get the minute the server started.  On some browsers, new CSS/JS
-###  doesn't load because the browser waits for 7 (or whatever) days
-###  to check for a new version.  This value is set by your web server,
-###  and it's normally a good thing to speed everything up.  However,
-###  when you upload new CSS/JS, you want all browsers to download the new
-###  files even if their cached version hasn't expired yet.
-###
-###  By adding an int to the end of the .css and .js files, browsers will
-###  see the files as *new* every time you restart your web server.
+###  Used as the default cgi_id.  See the documentation for
+###  StaticRenderer below for more information.
+
 SERVER_START_MINUTE = int(time.time() / 60)  # minutes since Jan 1, 1970
 
 
 #######################################################################
 ###   A dict of template renderers for scripts and styles in our apps.
-###   These are created as needed in TemplateInfo below.
+###   These are created as needed in TemplateInfo below and cached here.
+###   One for each app is created in each dict.
 
 SCRIPT_RENDERERS = {}
 STYLE_RENDERERS = {}
@@ -96,7 +94,7 @@ class TemplateInfo(object):
     # the static templatename.css file
     self.css = None
     if os.path.exists(os.path.join(self.app_dir, 'styles', self.template_name + '.css')):
-      self.css = '<link rel="stylesheet" type="text/css" href="%s?%i" />' % (os.path.join(settings.STATIC_URL, self.app, 'styles', self.template_name + '.css'), SERVER_START_MINUTE)
+      self.css = '<link rel="stylesheet" type="text/css" href="%s?{cgi_id}" />' % os.path.join(settings.STATIC_URL, self.app, 'styles', self.template_name + '.css')
     # the mako-rendered templatename.cssm file
     self.cssm = None
     if os.path.exists(os.path.join(self.app_dir, 'styles', self.template_name + '.cssm')):
@@ -104,7 +102,7 @@ class TemplateInfo(object):
     # the static templatename.js file
     self.js = None
     if os.path.exists(os.path.join(self.app_dir, 'scripts', self.template_name + '.js')):
-      self.js = '<script src="%s?%i"></script>' % (os.path.join(settings.STATIC_URL, self.app, 'scripts', self.template_name + '.js'), SERVER_START_MINUTE)
+      self.js = '<script src="%s?{cgi_id}"></script>' % os.path.join(settings.STATIC_URL, self.app, 'scripts', self.template_name + '.js')
     # the mako-rendered templatename.jsm file
     self.jsm = None
     if os.path.exists(os.path.join(self.app_dir, 'scripts', self.template_name + '.jsm')):
@@ -112,40 +110,76 @@ class TemplateInfo(object):
     
 
 class StaticRenderer(object):
-  '''The styles and scripts for a given template.'''
-  def __init__(self, mako_self):
-    # get the inheritance chain for this template
-    self.template_infos = []
-    while mako_self != None:
-      self.template_infos.insert(0, TemplateInfo(mako_self.template))  # go in reversed order so the most specialized template CSS/JS prints last and wins in a conflict
-      mako_self = mako_self.inherits
+  '''Renders the styles and scripts for a given template. 
+     
+     The mako_self parameter is simply the "self" variable
+     accessible within any Mako template.  An example call is:
+     
+     <%! from django_mako_plus.controller import static_files %>
+     <%  static_renderer = static_files.StaticRenderer(self) %>
+     
+     The optional cgi_id parameter is a less obvious.  On some browsers, 
+     new CSS/JS files don't load because the browser waits for 7 (or whatever) days
+     to check for a new version.  This value is set by your web server,
+     and it's normally a good thing to speed everything up.  However,
+     when you upload new CSS/JS, you want all browsers to download the new
+     files even if their cached versions have't expired yet.  
 
+     By adding an arbitrary id to the end of the .css and .js files, browsers will
+     see the files as *new* anytime that id changes.  The default method 
+     for calculating the id is the server start time (minutes since 1970). This
+     id automatically increments and never repeats as you restart your server.
+
+     This id works well in most installations, but it refreshes more than
+     necessary when servers restart themselves at given intervals
+     (such as uwsgi's max-requests option where the server restarts
+     after a specific number of requests).  If you are in this situation,
+     you'll have to send in the cgi_id.  When you want to signal that browsers
+     should redownload CSS/JS files from your site, change the id.  The way
+     you create and change the id is different in each situation.
+  '''
+  def __init__(self, mako_self, cgi_id=SERVER_START_MINUTE):
+    self.mako_self = mako_self
+    self.cgi_id = cgi_id
+    # step up the template inheritance chain and ensure each template has a TemplateInfo object
+    # I attach it to the template objects because they are cached by mako
+    while mako_self != None:
+      if settings.DEBUG or not hasattr(mako_self.template, DMP_ATTR_NAME):  # always recreate in debug mode
+        setattr(mako_self.template, DMP_ATTR_NAME, TemplateInfo(mako_self.template))
+      mako_self = mako_self.inherits
+    
 
   def get_template_css(self, request, context):
     '''Retrives the static and mako-rendered CSS'''
     ret = []
-    for ti in self.template_infos:
+    mako_self = self.mako_self
+    while mako_self != None:
+      ti = getattr(mako_self.template, DMP_ATTR_NAME)
       if ti.css:
-        ret.append(ti.css)  # the <link> was already created once in the constructor
+        ret.append(ti.css.format(cgi_id=self.cgi_id))  # the <link> was already created once in the constructor
       if ti.cssm:
         css_text = STYLE_RENDERERS[ti.app].render(request, ti.cssm, context.kwargs)
         if settings.DMP_MINIFY_JS_CSS and JSMIN:
           css_text = cssmin(css_text)
         ret.append('<style type="text/css">%s</style>' % css_text) 
+      mako_self = mako_self.inherits
     return '\n'.join(ret)
 
 
   def get_template_js(self, request, context):
     '''Retrieves the static and mako_rendered CSS'''    
     ret = []
-    for ti in self.template_infos:
+    mako_self = self.mako_self
+    while mako_self != None:
+      ti = getattr(mako_self.template, DMP_ATTR_NAME)
       if ti.js:
-        ret.append(ti.js)  # the <script> was already created once in the constructor
+        ret.append(ti.js.format(cgi_id=self.cgi_id))  # the <script> was already created once in the constructor
       if ti.jsm:
         js_text = SCRIPT_RENDERERS[ti.app].render(request, ti.jsm, context.kwargs)
         if settings.DMP_MINIFY_JS_CSS and JSMIN:
           js_text = jsmin(js_text)
         ret.append('<script>%s</script>' % js_text)
+      mako_self = mako_self.inherits
     return '\n'.join(ret)
 
 
